@@ -463,9 +463,9 @@ void configure_r_port(struct r_port *info)
 	if (cflag & PARENB) {
 		sEnParity(cp);
 		if (cflag & PARODD) {
-			sSetEvenParity(cp);
-		} else {
 			sSetOddParity(cp);
+		} else {
+			sSetEvenParity(cp);
 		}
 	} else {
 		sDisParity(cp);
@@ -765,6 +765,7 @@ static void rp_close(struct tty_struct *tty, struct file * filp)
 			break;
 	}
 	
+	xmit_flags[info->line >> 5] &= ~(1 << (info->line & 0x1f));
 	sDisTransmit(cp);
 	sDisInterrupts(cp, (TXINT_EN|MCINT_EN|RXINT_EN|SRCINT_EN|CHANINT_EN));
 	sDisCTSFlowCtl(cp);
@@ -1141,6 +1142,7 @@ void rp_hangup(struct tty_struct *tty)
 		rp_num_ports_open--;
 	}
 	
+	xmit_flags[info->line >> 5] &= ~(1 << (info->line & 0x1f));
 	info->count = 0;
 	info->flags &= ~(ROCKET_NORMAL_ACTIVE|ROCKET_CALLOUT_ACTIVE);
 	info->tty = 0;
@@ -1241,7 +1243,7 @@ static int rp_write(struct tty_struct * tty, int from_user,
 		info->xmit_fifo_room -= c;
 	}
 	if (!count)
-		return written;
+		goto end;
 	
 	save_flags(flags);
 	while (1) {
@@ -1272,6 +1274,13 @@ static int rp_write(struct tty_struct * tty, int from_user,
 	}
 	xmit_flags[info->line >> 5] |= (1 << (info->line & 0x1f));
 	restore_flags(flags);
+end:
+	if (info->xmit_cnt < WAKEUP_CHARS) {
+		if ((tty->flags & (1 << TTY_DO_WRITE_WAKEUP)) &&
+		    tty->ldisc.write_wakeup)
+			(tty->ldisc.write_wakeup)(tty);
+		wake_up_interruptible(&tty->write_wait);
+	}
 	return written;
 }
 
@@ -1349,6 +1358,7 @@ init_module( void) {
 	int aiop, chan;
 	unsigned int aiopio[MAX_AIOPS_PER_BOARD];
 	CONTROLLER_t *ctlp;
+	int	reserved_controller = 0;
 
 	printk("Rocketport device driver module, version %s, %s\n",
 	       ROCKET_VERSION, ROCKET_DATE);
@@ -1376,6 +1386,11 @@ init_module( void) {
 		board1 = 0x180;
 	if (controller <= 0)
 		controller = board1 + 0x40;
+
+	if (check_region(controller, 4)) {
+		printk("Controller IO addresses in use, unloading driver.\n");
+		return -EBUSY;
+	}
 	
 	rcktpt_io_addr[0] = board1;
 	rcktpt_io_addr[1] = board2;
@@ -1389,15 +1404,24 @@ init_module( void) {
 	for (i=0; i < NUM_BOARDS; i++) {
 		if (rcktpt_io_addr[i] <= 0)
 			continue;
-		
+
+		if (check_region(rcktpt_io_addr[i], 64)) {
+			printk("Rocketport board address 0x%x in use...\n",
+			       rcktpt_io_addr[i]);
+			rcktpt_io_addr[i] = 0;
+			continue;
+		}
+	
 		for (aiop=0; aiop<MAX_AIOPS_PER_BOARD; aiop++)
 			aiopio[aiop]= rcktpt_io_addr[i] + (aiop * 0x400);
 		ctlp= sCtlNumToCtlPtr(i);
 		num_aiops = sInitController(ctlp, i, controller + (i*0x400),
 					    aiopio, MAX_AIOPS_PER_BOARD, 0,
 					    FREQ_DIS, 0);
-		if (num_aiops <= 0)
+		if (num_aiops <= 0) {
+			rcktpt_io_addr[i] = 0;
 			continue;
+		}
 		num_ports = 0;
 
 		for (aiop = 0; aiop < num_aiops; aiop++) {
@@ -1413,6 +1437,14 @@ init_module( void) {
 		       num_aiops, num_ports);
 		max_board = i;
 		total_ports += num_ports;
+		if (rcktpt_io_addr[i] + 0x40 == controller) {
+			reserved_controller++;
+			request_region(rcktpt_io_addr[i], 68,
+				       "Comtrol Rocketport");
+		} else {
+			request_region(rcktpt_io_addr[i], 64,
+				       "Comtrol Rocketport");
+		}
 	}
 
 	if (total_ports == 0) {
@@ -1421,6 +1453,9 @@ init_module( void) {
 		return -ENODEV;
 	}
 
+	if (reserved_controller == 0)
+		request_region(controller, 4, "Comtrol Rocketport");
+
 	/*
 	 * Set up the tty driver structure and then register this
 	 * driver with the tty layer.
@@ -1491,6 +1526,7 @@ void
 cleanup_module( void) {
 	int	retval;
 	int	i;
+	int	released_controller = 0;
 	
 	retval = tty_unregister_driver(&callout_driver);
 	if (retval) {
@@ -1506,6 +1542,17 @@ cleanup_module( void) {
 		if (rp_table[i])
 			kfree(rp_table[i]);
 	}
+	for (i=0; i < NUM_BOARDS; i++) {
+		if (rcktpt_io_addr[i] <= 0)
+			continue;
+		if (rcktpt_io_addr[i] + 0x40 == controller) {
+			released_controller++;
+			release_region(rcktpt_io_addr[i], 68);
+		} else
+			release_region(rcktpt_io_addr[i], 64);
+		if (released_controller == 0)
+			release_region(controller, 4);
+	}
 	if (tmp_buf)
 		free_page((unsigned long) tmp_buf);
 	timer_table[COMTROL_TIMER].fn = 0;
